/*
    Watch Dog Adapter Test/Startup Program ---------------- DECISION Taiwan
    This is a sample C program to control WatchDog Adapter.
    You can use Borland C, MicroSoft C ... to compiler it.
    Usage :  WatchDog [ Auto [ IOAddr [ Timer T ]]]
    Auto : 1 = Auto-Refresh, 0 = Manual Refresh
    IOAddr : the base address of WatchDog Adapter
	     legal value : 0x000 to 0xFFE
	Adapter Enable/Refresh  ==> read I/O port with address IOAddr
	Adapter Disable         ==> read I/O port with address IOAddr+1
	Remote Power On         ==> write 0 to I/O port with address IOAddr+1
	Remote Power Off        ==> write 1 to I/O port with address IOAddr+1
    Timer : Refresh Timer = 62.5ms * pow(2,T)
    Example : WatchDog 1 0xBBC 5
       Enable WatchDog at address 0xBBC to check system automatically.
       System will reboot if system is hanged more than 1 seconds.
*/
#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <time.h>
#include <math.h>

#define WdgDefaultAuto   1
#define WdgDefaultAddr   0xBBC
#define WdgDefaultTime   5

void watchdog(int);                // Open the device
void watchdog_refresh(int);        // refresh the device again
void disable_watchdog(int);        // disable the device and Close it
void watchdog_control(int, int);   // Output One Command to device

void main(int argc, char *argv[])
{
    unsigned int WdgAuto = WdgDefaultAuto;
    unsigned int WdgAddr = WdgDefaultAddr;
    unsigned int WdgTime = WdgDefaultTime;

    int     DoWork=1;   // Command Input from Keyboard
    float   WdgRefresh;
    clock_t ClockStart;
    clock_t ClockPeriod;

    clrscr();
    if ( ( argc == 2 ) || ( argc == 3 ) || (argc == 4 ) ) {
	if ( ( argv[1][0] == '0' ) || ( argv[1][0] == '1' ) ) {
	    if ( strlen(argv[1]) != 1 ) argc = 99;
	    else WdgAuto = atoi(argv[1]);
	} else argc = 99;
    }
    if ( ( argc == 3 ) || (argc == 4 ) ) {
	if ( sscanf(argv[2], "%x", &WdgAddr) <= 0 ) argc = 99;
	if ( WdgAddr >= 0xFFF ) argc = 99;
    }
    if ( argc == 4 ) {
	WdgTime = atoi(argv[3]);
	if ( ( WdgTime<= 0 ) || ( WdgTime > 11 ) ) argc = 99;
    }
    if ( argc > 4 ) {
	if ( argc == 99 ) {
	    fprintf(stderr, "Input Command Error\n");
	}
	printf("Usage : WatchDog [Auto(0/1) [IOAddr [Timer(1-11)]]]\n");
	exit(0);
    }
    WdgRefresh = 62.5*pow(2,WdgTime-1);
    if ( WdgAuto ) {
	ClockStart = clock();
	printf("WatchDog is Auto enable Addr=0x%03x Time=%6.1lfms\n", WdgAddr, WdgRefresh);
	printf("   Press 'Q' to stop watchdog\n");
	printf("         'N' to enable Remote Power\n");
	printf("         'F' to disable Remote Power\n");
    } else {
	printf("WatchDog is manual controlled, Addr=0x%03x Time=%6.1lfms\n", WdgAddr, WdgRefresh);
	printf("Input the Command : ( [Q]uit to Exit ) \n");
	printf("   WatchDog [E]nable [D]isable [R]efresh\n");
	printf("   Remote Power Control of[F]   o[N]\n");
    }

    while ( DoWork ) {
	if ( WdgAuto ) {
	    ClockPeriod = clock() - ClockStart;
	    if ( (ClockPeriod*1000/CLK_TCK) > (WdgRefresh/2.0) ) {
		watchdog_refresh(WdgAddr);
		ClockStart = clock();
	    }
	}
	if ( ! kbhit() ) continue;
	switch ( getch() ) {
	case ('Q'):
	case ('q'): DoWork = 0; break;
	case ('E'):
	case ('e'): if (WdgAuto) continue;
		    watchdog(WdgAddr);            break;
	case ('D'):
	case ('d'): if (WdgAuto) continue;
		    disable_watchdog(WdgAddr);    break;
	case ('F'):
	case ('f'): watchdog_control(WdgAddr, 1); break;
	case ('n'): watchdog_control(WdgAddr, 0); break;
	case ('R'):
	case ('r'): if (WdgAuto) continue;
		    watchdog_refresh(WdgAddr);    break;
	}
    }
    disable_watchdog(WdgAddr);
}

void watchdog(int Addr)
{
    if ( Addr >= 0xFFF ) return;
    inportb((unsigned int)Addr);
}

void watchdog_refresh(int Addr)
{
    if ( Addr >= 0xFFF ) return;
    inportb((unsigned int)Addr);
}

void disable_watchdog(int Addr)
{
    if ( Addr >= 0xFFF ) return;
    inportb((unsigned int)Addr+1);
}

void watchdog_control(int Addr, int Value)
{
    if ( Value == 0 ) outportb((unsigned int)(Addr+1), 0);
    else outportb((unsigned int)(Addr+1), 1);
}